﻿/*
 * Read iTunes Mediathek Library (C) Daniel Kirstenpfad 2008
 * 
 * Licensed under Attribution-NonCommercial-ShareAlike 3.0 Unported license (see license.txt)
 * 
 * 
 * 
 * */

using System;
using System.IO;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;


namespace WebTunes
{
    class readiTunesMediathek
    {
        public static List<Song> Songs = new List<Song>();

        public static String ReadNextElement(XmlTextReader reader)
        {
            reader.Read();
            reader.Read();
            if (reader.Name == "true") return reader.Name;
            if (reader.Name == "false") return reader.Name;
            reader.Read();
            String Value = reader.Value;
            reader.Read();
            return Value;
        }

        static void Main(string[] args)
        {
            XmlTextReader reader = new XmlTextReader("Mediathek.xml");

            #region jump over the first two <dict> tags..
            int counter = 0;

            while (reader.Read())
            {
                switch (reader.NodeType)
                {
                    case XmlNodeType.Element:
                        switch (reader.Name)
                        {
                            case "dict":
                                counter++;
                                break;
                        }
                        break;
                }
                if (counter == 2) break;
            }
            #endregion

            Song newSong = new Song();

            #region Read the data...
            try
            {
                while (reader.Read())
                {
                    switch (reader.NodeType)
                    {
                        case XmlNodeType.Element:
                            switch (reader.Name)
                            {
                                #region Nodes
                                case "key":
                                    reader.Read();
                                    switch (reader.Value)
                                    {
                                        case "Track ID":
                                            newSong.TrackID = Convert.ToInt64(ReadNextElement(reader));
                                            break;
                                        case "Name":
                                            newSong.Name = ReadNextElement(reader);
                                            break;
                                        case "Artist":
                                            newSong.Artist = ReadNextElement(reader);
                                            break;
                                        case "Album Artist":
                                            newSong.AlbumArtist = ReadNextElement(reader);
                                            break;
                                        case "Composer":
                                            newSong.Composer = ReadNextElement(reader);
                                            break;
                                        case "Album":
                                            newSong.Album = ReadNextElement(reader);
                                            break;
                                        case "Genre":
                                            newSong.Genre = ReadNextElement(reader);
                                            break;
                                        case "Kind":
                                            newSong.Kind = ReadNextElement(reader);
                                            break;
                                        case "Size":
                                            newSong.Size = Convert.ToInt64(ReadNextElement(reader));
                                            break;
                                        case "Total Time":
                                            newSong.TotalTime = Convert.ToInt32(ReadNextElement(reader));
                                            break;
                                        case "Year":
                                            newSong.Year = Convert.ToInt32(ReadNextElement(reader));
                                            break;
                                        case "Date Modified":
                                            DateTime.TryParse(ReadNextElement(reader), out newSong.Modified);
                                            break;
                                        case "Date Added":
                                            DateTime.TryParse(ReadNextElement(reader), out newSong.Added);
                                            break;
                                        case "Bit Rate":
                                            newSong.Bitrate = Convert.ToInt32(ReadNextElement(reader));
                                            break;
                                        case "Sample Rate":
                                            newSong.SampleRate = Convert.ToInt32(ReadNextElement(reader));
                                            break;
                                        case "Comments":
                                            newSong.Comments = ReadNextElement(reader);
                                            break;
                                        case "Play Count":
                                            newSong.PlayCount = Convert.ToInt64(ReadNextElement(reader));
                                            break;
                                        case "Play Date UTC":
                                            DateTime.TryParse(ReadNextElement(reader), out newSong.PlayDate);
                                            break;
                                        case "Release Date":
                                            DateTime.TryParse(ReadNextElement(reader), out newSong.ReleaseDate);
                                            break;
                                        case "Artwork Count":
                                            newSong.ArtworkCount = Convert.ToInt32(ReadNextElement(reader));
                                            break;
                                        case "Podcast":
                                            newSong.Podcast = Convert.ToBoolean(ReadNextElement(reader));
                                            break;
                                        case "Compilation":
                                            newSong.Compilation = Convert.ToBoolean(ReadNextElement(reader));
                                            break;
                                        case "Location":
                                            newSong.Location = ReadNextElement(reader);
                                            break;
                                        case "Rating":
                                            newSong.Rating = Convert.ToInt16(ReadNextElement(reader));
                                            break;
                                    }
                                    break;
                                #endregion
                            }
                            break;
                        case XmlNodeType.EndElement: // End-Elements...
                            switch (reader.Name)
                            {
                                // we waited for this one...
                                case "dict":
                                    Songs.Add(newSong);
                                    newSong = new Song();
                                    break;
                            }
                            break;
                    }
                }
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }
            #endregion

            Console.WriteLine("Song Count: " + Songs.Count);

            #region Write Out to XML File
            FileStream fs = new FileStream("webtunes.mediathek.xml", FileMode.Create, FileAccess.Write);
            try
            {
                System.Xml.Serialization.XmlSerializer xmls = new XmlSerializer(Songs.GetType());
                xmls.Serialize(fs, Songs);
            }
            finally
            {
                fs.Close();
            }
            #endregion
        }
    }
}
